@@ -1,6 +1,7 @@
 # Orca
 #
-# Copyright 2004-2008 Sun Microsystems Inc.
+# Copyright 2004-2009 Sun Microsystems Inc.
+# Copyright 2010 Joanmarie Diggs, Mesar Hameed
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -22,7 +23,8 @@
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
+__copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc." \
+                "Copyright (c) 2010 Joanmarie Diggs, Mesar Hameed."
 __license__   = "LGPL"
 
 # We're going to force the name of the app to "orca" so pygtk
@@ -57,6 +59,7 @@ except RuntimeError:
 
 import getopt
 import os
+import re
 import signal
 import time
 import unicodedata
@@ -368,11 +371,33 @@ def exitLearnMode(self, inputEvent=None):
     # mode.
     #
     message = _("Exiting learn mode.")
-    speech.speak(message)
-    braille.displayMessage(message)
+    orca_state.activeScript.presentMessage(message)
     settings.learnModeEnabled = False
     return True
 
+def exitListShortcutsMode(self, inputEvent=None):
+    """Turns list shortcuts mode off.
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    orca_state.listOfShortcuts = []
+    orca_state.typeOfShortcuts = ""
+    orca_state.ptrToShortcut = -1
+    settings.listShortcutsModeEnabled = False
+
+    # Translators: Orca has a "List Shortcuts Mode" that allows the user to
+    # list a group of keyboard shortcuts. Pressing 1 makes it possible for
+    # the user to navigate amongst a list of global ("default") commands.
+    # Pressing 2 allows the user to navigate amongst Orca commands specific
+    # to the application with focus. Escape exists this mode. This string
+    # is the prompt which will be presented to the user in both speech and
+    # braille upon exiting this mode.
+    #
+    message = _("Exiting list shortcuts mode.")
+    orca_state.activeScript.presentMessage(message)
+    return True
+
 ########################################################################
 #                                                                      #
 # METHODS FOR PRE-PROCESSING AND MASSAGING KEYBOARD EVENTS.            #
@@ -665,7 +690,7 @@ def keyEcho(event):
             eventType == KeyEventType.PRINTABLE \
             and not _orcaModifierPressed \
             and orca_state.activeScript \
-            and orca_state.activeScript.willEchoCharacter(event)
+            and orca_state.activeScript.utilities.willEchoCharacter(event)
 
         # One last check for echoing -- a PRINTABLE key may have squeaked
         # through due to the enableEchoByCharacter check above.  We only
@@ -877,7 +902,11 @@ def _processKeyboardEvent(event):
         if orca_state.capturingKeys:
             consumed = _processKeyCaptured(keyboardEvent)
         else:
-            consumed = _keyBindings.consumeKeyboardEvent(None, keyboardEvent)
+            if settings.listShortcutsModeEnabled:
+                consumed = listShortcuts(keyboardEvent)
+            if (not consumed):
+                consumed = _keyBindings.consumeKeyboardEvent( \
+                  None, keyboardEvent)
             if (not consumed) and (_currentPresentationManager >= 0):
                 consumed = _PRESENTATION_MANAGERS[_currentPresentationManager].\
                            processKeyboardEvent(keyboardEvent)
@@ -963,12 +992,12 @@ def toggleSilenceSpeech(script=None, inputEvent=None):
         # Translators: this is a spoken prompt letting the user know
         # that speech synthesis has been turned back on.
         #
-        speech.speak(_("Speech enabled."))
+        orca_state.activeScript.presentMessage(_("Speech enabled."))
     else:
         # Translators: this is a spoken prompt letting the user know
         # that speech synthesis has been temporarily turned off.
         #
-        speech.speak(_("Speech disabled."))
+        orca_state.activeScript.presentMessage(_("Speech disabled."))
         settings.silenceSpeech = True
     return True
 
@@ -1092,12 +1121,41 @@ def loadUserSettings(script=None, inputEvent=None):
     # For now, we'll just look at the Orca modifier keys we support
     # (Caps Lock, KP_Insert, and Insert).]]]
     #
+    clearedMods = {}
+    cmd = ""
     for keyName in settings.orcaModifierKeys:
-        if keyName == "Caps_Lock":
-            os.system('xmodmap -e "clear Lock"')
+        # We don't want an X Windows System modifier key to change the
+        # X modifier if it is used as the Orca modifier key.
+        #
+        if keyName in orca_state.xmodmapMods:
+            orca_state.clearedXmodmapMods[keyName] = True
+            clearedMods[keyName] = True
+
+        # In addition, if the KP_Insert key is used as the Orca
+        # modifier key, we want to make sure we clear any other
+        # keysyms that might be in use on that key since we won't be
+        # able to detect them as being the Orca modifier key.  For
+        # example, KP_Insert produces "KP_Insert" when pressed by
+        # itself, but Shift+KP_Insert produces "0". The original
+        # values are saved/reset in the orca shell script.
+        #
         if keyName in ["Caps_Lock", "KP_Insert", "Insert"]:
-            command = 'xmodmap -e "keysym %s = %s"' % (keyName, keyName)
-            os.system(command)
+            cmd += "keysym %s = %s\n" % (keyName, keyName)
+
+    modifyXmodmapMods()
+
+    # If some keys were previously the Orca key and were also
+    # originally a normal X Window System modifier, we may need to
+    # re-enable them as modifiers if they are no longer the Orca key.
+    #
+    for i in orca_state.clearedXmodmapMods:
+        if orca_state.clearedXmodmapMods[i] and i not in clearedMods:
+            cmd += "add " + orca_state.xmodmapMods[i] + " = " + i + "\n"
+            orca_state.clearedXmodmapMods[i] = False
+    if cmd:
+        cmd = "echo \"" + cmd + "\" | xmodmap - >/dev/null 2>&1"
+        os.system(cmd)
+        debug.println(debug.LEVEL_FINEST,"%s" % cmd)
 
     if _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].activate()
@@ -1199,6 +1257,217 @@ def helpForOrca(script=None, inputEvent=None):
                  gtk.get_current_event_time())
     return True
 
+def listShortcuts(event):
+    """When list shortcuts mode is enabled, this function provides a means
+    by which users can navigate through Orca bound Orca commands. Pressing
+    1 results in a list of the default shortcuts; pressing 2 results in a
+    list of shortcuts for the focused application, should one exist. List
+    navigation is accomplished through the Up and Down Arrow keys. Escape
+    exits the list. In this mode, other keys are disabled.
+
+    Arguments:
+    - event: an AT-SPI DeviceEvent
+
+    Returns True if the event is consumed (and False if not).
+    """
+
+    numShortcuts = len(orca_state.listOfShortcuts)
+    consumed = False
+    clickCount = 0
+    message = ""
+
+    # Translators: The following string instructs the user how to navigate
+    # amongst the list of commands presented in 'list shortcuts' mode as
+    # well as how to exit the list when finished.
+    #
+    navigation = \
+        _("Use Up and Down Arrow to navigate the list. Press Escape to exit.")
+
+    if event.type == pyatspi.KEY_PRESSED_EVENT:
+        clickCount = orca_state.activeScript.getClickCount()
+        if (event.event_string == "1"):
+            if not numShortcuts or orca_state.typeOfShortcuts != "default":
+                orca_state.listOfShortcuts = getListOfShortcuts("default")
+                orca_state.typeOfShortcuts = "default"
+                numShortcuts = len(orca_state.listOfShortcuts)
+            orca_state.ptrToShortcut = 0
+            # Translators: This message is presented when the user is in
+            # 'list of shortcuts mode'. In this messsage, we present the
+            # number of shortcuts found.
+            #
+            message = _("%d Orca default shortcuts found.") % numShortcuts
+            message = message + " " + navigation
+            orca_state.activeScript.presentMessage(message)
+            message = orca_state.listOfShortcuts[orca_state.ptrToShortcut][0]+ \
+              " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
+            speech.speak(message)
+            orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+        elif (event.event_string == "2"):
+            if not numShortcuts or orca_state.typeOfShortcuts != "application":
+                orca_state.listOfShortcuts = getListOfShortcuts("application")
+                orca_state.typeOfShortcuts = "application"
+                numShortcuts = len(orca_state.listOfShortcuts)
+            if numShortcuts > 0: 
+                orca_state.ptrToShortcut = 0
+                # Translators: This message is presented when the user is in
+                # 'list of shortcuts mode'. In this message, we present the
+                # number of shortcuts found for the named application.
+                #
+                message = \
+                    _("%(count)d Orca shortcuts for %(application)s found.") % \
+                     {"count" : numShortcuts,
+                      "application" : orca_state.activeScript.app.name}
+                message = message + " " + navigation
+                orca_state.activeScript.presentMessage(message)
+                message = \
+                  orca_state.listOfShortcuts[orca_state.ptrToShortcut][0] + \
+                  " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
+                speech.speak(message)
+                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            else:
+                # Translators: This message is presented when the user is in
+                # 'list of shortcuts mode'. This is the message we present
+                # when the user requested a list of application-specific
+                # shortcuts, but none could be found for that application.
+                #
+                message = _("No Orca shortcuts for %s found.") % \
+                    (orca_state.activeScript.app.name)
+                speech.speak(message)
+                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+        elif (event.event_string == "Up"):
+            if (numShortcuts > 0):
+                if orca_state.ptrToShortcut > 0: 
+                    orca_state.ptrToShortcut = orca_state.ptrToShortcut-1
+                else:
+                    orca_state.ptrToShortcut = numShortcuts-1 
+                    # Translators: when the user is attempting to locate a
+                    # particular object and the top of a page or list is
+                    # reached without that object being found, we "wrap" to
+                    # the bottom and continue looking upwards. We need to
+                    # inform the user when this is taking place.
+                    #
+                    orca_state.activeScript.\
+                        presentMessage(_("Wrapping to bottom."))
+
+                message = \
+                  orca_state.listOfShortcuts[orca_state.ptrToShortcut][0] + \
+                  " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
+                speech.speak(message)
+                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+        elif (event.event_string == "Down"):
+            if (numShortcuts > 0): 
+                if orca_state.ptrToShortcut < numShortcuts-1: 
+                    orca_state.ptrToShortcut = orca_state.ptrToShortcut+1
+                else:
+                    orca_state.ptrToShortcut = 0 
+                    # Translators: when the user is attempting to locate a
+                    # particular object and the bottom of a page or list is
+                    # reached without that object being found, we "wrap" to the
+                    # top and continue looking downwards. We need to inform the
+                    # user when this is taking place.
+                    #
+                    orca_state.activeScript.\
+                        presentMessage(_("Wrapping to top."))
+                message = \
+                  orca_state.listOfShortcuts[orca_state.ptrToShortcut][0] + \
+                  " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
+                speech.speak(message)
+                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+        elif (event.event_string == "Escape"):
+            exitListShortcutsMode(event)
+            consumed = True 
+        else:
+            # Translators: Orca has a 'List Shortcuts' mode by which a user can
+            # navigate through a list of the bound commands in Orca. Pressing 1
+            # presents the commands/shortcuts available for all applications.
+            # These are the "default" commands/shortcuts. Pressing 2 presents
+            # commands/shortcuts Orca provides for the application with focus.
+            # The following message is presented to the user upon entering this
+            # mode.
+            #
+            message = _("Press 1 for Orca's default shortcuts. Press 2 for " \
+                        "Orca's shortcuts for the current application. " \
+                        "Press escape to exit.")
+            speech.speak(message)
+            orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+    elif (event.type == pyatspi.KEY_RELEASED_EVENT) and (event.event_string \
+      == "Escape"):
+        consumed = True
+    return consumed
+
+
+def getListOfShortcuts(typeOfShortcuts):
+    """This function returns a list of Orca default shortcuts if the argument
+    is 'default'. It returns a list of Orca shortcuts which are specific to
+    the focused application, if the argument is "application". Orca default
+    shortcuts are those found in the default script. Application-specific
+    shortcuts are those which are present in the active script, but not in
+    the default script. Only one shortcut per handler is listed. The list is
+    sorted on shortcuts.
+
+    Arguments:
+    - typeOfShortcuts: a string specifying the desired type of shortcuts.
+
+    Returns a list of shortcuts; depending on the value of argument.
+    """
+
+    import default
+
+    numShortcuts = len(orca_state.listOfShortcuts)
+    shortcuts = []
+    shortcut = ""
+    clickCount = ""
+    brlKeyName = ""     
+    brlHandler = None
+    defScript = default.Script(None)
+    defKeyBindings = defScript.getKeyBindings()
+    defBrlBindings = defScript.getBrailleBindings()
+    kbindings = keybindings.KeyBindings()
+    if typeOfShortcuts == "default":
+        for kb in defKeyBindings.keyBindings:
+            if kb.keysymstring:
+                if not kbindings.hasKeyBinding(kb,"description"):
+                    kbindings.add(kb)
+    elif typeOfShortcuts == "application":
+        for kb in orca_state.activeScript.keyBindings.keyBindings:
+            if kb.keysymstring:
+                if not (defKeyBindings.hasKeyBinding(kb,"description") or \
+                kbindings.hasKeyBinding(kb,"description")):
+                    kbindings.add(kb)
+
+    for kb in kbindings.keyBindings:
+        keysymString = kb.keysymstring.replace("KP_", _("keypad "))
+        clickCount = ""
+        if kb.click_count == 2:
+            # Translators: Orca keybindings support double
+            # and triple "clicks" or key presses, similar to
+            # using a mouse.
+            #
+            clickCount = _("double click")
+        elif kb.click_count == 3:
+            # Translators: Orca keybindings support double
+            # and triple "clicks" or key presses, similar to
+            # using a mouse.
+            #
+            clickCount = _("triple click")
+        shortcut = (kb.handler.description, keybindings.getModifierNames\
+        (kb.modifiers) + keysymString.upper() + " " + clickCount)
+        shortcuts.append(shortcut)
+        shortcuts = sorted(shortcuts, key=lambda shortcut: shortcut[1])
+
+    if typeOfShortcuts == "orcaDefault":
+        for bb in defBrlBindings.keys():
+            brlKeyName = braille.command_name[bb]
+            brlHandler = defBrlBindings[bb]
+            shortcut = (brlHandler.description, brlKeyName)
+            shortcuts.append(shortcut) 
+    return shortcuts
+
 def quitOrca(script=None, inputEvent=None):
     """Quit Orca. Check if the user wants to confirm this action.
     If so, show the confirmation GUI otherwise just shutdown.
@@ -1327,6 +1596,10 @@ def start(registry):
 
 def die(exitCode=1):
 
+    # Restore any xmodmap modifiers that we might have changed.
+    #
+    restoreXmodmapMods()
+
     # We know what we are doing here, so tell pylint not to flag
     # the _exit method call as a warning.  The disable-msg is
     # localized to just this method.
@@ -1390,6 +1663,11 @@ def shutdown(script=None, inputEvent=None):
         signal.alarm(0)
 
     _initialized = False
+
+    # Restore any xmodmap modifiers that we might have changed.
+    #
+    restoreXmodmapMods()
+
     return True
 
 exitCount = 0
@@ -1544,6 +1822,15 @@ def usage():
     print "-q, --quit                   " + \
           _("Quits Orca (if shell script used)")
 
+    # Translators: this is the Orca command line option that will force 
+    # the termination of Orca.
+    # The user would run the Orca shell script again from a terminal window.
+    # If this command line option is specified, the script will quit any
+    # instances of Orca that are already running.
+    #
+    print "-f, --forcequit              " + \
+          _("Forces orca to be terminated immediately.")
+
     # Translators: this is the Orca command line option to tell Orca to
     # replace any existing Orca process(es) that might be running.
     #
@@ -1745,6 +2032,9 @@ def main():
         print "environment variable has been set."
         return 1
 
+    # Save the current xmodmap modifiers before we change anything.
+    storeXmodmapMods()
+
     userprefs = settings.userPrefsDir
     sys.path.insert(0, userprefs)
     sys.path.insert(0, '') # current directory
@@ -1782,5 +2072,54 @@ def main():
     start(pyatspi.Registry) # waits until we stop the registry
     return 0
 
+
+def storeXmodmapMods():
+    """ Save the state of the xmodmap modifiers before we customize them. """
+
+    cmd = "xmodmap | tail -n 9 | head -n 8 | sed " + \
+    "-e 's/([^)]*)//g' -e 's/,//g'"
+    filehandle = os.popen(cmd)
+    txt = filehandle.read()
+    filehandle.close()
+    lines = txt.splitlines()
+    for i in range(0, len(lines)):
+        lines[i] = re.sub('\ +', ' ', lines[i])
+        lines[i] = re.sub('\ +$', '', lines[i])
+        lst = lines[i].split(" ")
+        tag = lst[0]
+        for i in lst[1:]:
+            orca_state.xmodmapMods[i] = tag
+    return
+
+def modifyXmodmapMods():
+    """ Modify the xmodmap modifiers to suit our orca modifier keys. """
+
+    return _xmodmapMods("remove ")
+
+def restoreXmodmapMods():
+    """ Restore the xmodmap modifiers to their initial state, before we 
+        changed them to suit our orca modifier keys.
+    """
+
+    return _xmodmapMods("add ")
+  
+def _xmodmapMods(keyWord):
+
+    # add/remove any xmodmap modifiers that we might have changed.
+    #
+    cmd = ""
+    # We know what we are doing,
+    # disable warning about i being possibly undefined.
+    # pylint: disable-msg=W0631
+    #
+    for i in orca_state.clearedXmodmapMods:
+        if orca_state.clearedXmodmapMods[i]:
+            cmd += keyWord + orca_state.xmodmapMods[i] + " = " + i + "\n"
+    if cmd:
+        cmd = "echo \"" + cmd + "\" | xmodmap - >/dev/null 2>&1"
+        os.system(cmd)
+        debug.println(debug.LEVEL_FINEST,"%s" % cmd)
+    return
+
 if __name__ == "__main__":
     sys.exit(main())
