@@ -33,7 +33,7 @@ ngettext = gettext.ngettext
 
 # Tell gettext where to find localized strings.
 #
-localedir = os.path.join ("/home/wwalker/git.gnome.org/orca/bld", "share", "locale")
+localedir = os.path.join ("/export/home/jd/orca/bld", "share", "locale")
 gettext.bindtextdomain ("orca", localedir)
 gettext.textdomain("orca")
 
