@@ -724,6 +724,15 @@ class Script(default.Script):
             #
             allTextFields = self.utilities.descendantsWithRole(
                 chatRoomTab, pyatspi.ROLE_TEXT, False)
+
+            # Pidgin 2.7 includes a Find text object, hidden by default.
+            # We can identify (and eliminate) it by STATE_SINGLE_LINE.
+            #
+            allTextFields = \
+                filter(lambda o: 
+                       not o.getState().contains(pyatspi.STATE_SINGLE_LINE),
+                       allTextFields)
+
             index = len(allTextFields) - 2
             if index >= 0:
                 self.chatAreas[chatRoomTab] = allTextFields[index]
@@ -744,6 +753,7 @@ class Script(default.Script):
                 self.chatRoomMessages[chatRoomName].append("")
                 i += 1
 
+        lastKey, mods = self.utilities.lastKeyAndModifiers()
         if event.source and (event.source == chatArea):
             # We always automatically go back to focus tracking mode when
             # someone sends us a message.
@@ -800,10 +810,7 @@ class Script(default.Script):
 
                 self.chatRoomMessages[chatRoomName].append(message)
 
-        elif isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent) \
-             and orca_state.lastNonModifierKeyEvent \
-             and (orca_state.lastNonModifierKeyEvent.event_string == "Tab") \
-             and event.any_data and (event.any_data != "\t"):
+        elif lastKey == "Tab" and event.any_data and event.any_data != "\t":
             # This is autocompleted text (the name of a user in an IRC
             # chatroom).  The default script isn't announcing it because
             # it's not selected.
