@@ -191,6 +191,11 @@ userCustomizableSettings = [
     "chatPrefixMessageWithName",
     "chatAnnounceBuddyTyping",
     "chatSeparateRoomHistories",
+    "enableFlashMessages",
+    "brailleFlashTime",
+    "flashIsPersistent",
+    "flashVerbosityLevel",
+    "messageVerbosityLevel",
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -250,6 +255,8 @@ VERBOSITY_LEVEL_BRIEF   = 0
 VERBOSITY_LEVEL_VERBOSE = 1
 speechVerbosityLevel    = VERBOSITY_LEVEL_VERBOSE
 brailleVerbosityLevel   = VERBOSITY_LEVEL_VERBOSE
+flashVerbosityLevel     = VERBOSITY_LEVEL_VERBOSE
+messageVerbosityLevel   = VERBOSITY_LEVEL_VERBOSE
 
 BRAILLE_ROLENAME_STYLE_SHORT = 0 # three letter abbreviations
 BRAILLE_ROLENAME_STYLE_LONG  = 1 # full rolename
@@ -356,6 +363,7 @@ silenceSpeech           = False
 #
 speechFactoryModules    = ["espeechfactory", \
                            "gnomespeechfactory", \
+                           "openttsfactory", \
                            "speechdispatcherfactory"]
 speechServerFactory     = "gnomespeechfactory"
 speechServerInfo        = None # None means let the factory decide.
@@ -390,10 +398,20 @@ enableBrailleContext    = True
 #
 enableBrailleGrouping   = False
 
+# If True, enable braille flash messages. Note that braille or braille
+# monitor will also need to be enabled for this setting to work.
+#
+enableFlashMessages     = True
+
 # The timeout (in milliseconds) to use for messages flashed in braille.
 #
 brailleFlashTime        = 5000
 
+# If True, flash messages should continue to be displayed until some
+# other message comes along, or the user presses a key.
+#
+flashIsPersistent       = False
+
 # If True, enable braille monitor.
 #
 enableBrailleMonitor    = False
@@ -575,7 +593,7 @@ enableEchoByCharacter   = False
 #
 enableEchoByWord        = False
 
-# if True, enable word echo.
+# if True, enable Sentence echo.
 # Note that it is allowable for both enableEchoByWord and enableEchoBySentence
 # to be True.
 #
@@ -823,6 +841,11 @@ cacheAccessibles        = True
 #
 learnModeEnabled        = False
 
+# Assists with list shortcuts mode (what you enter when you press 
+# Insert + H (double click) and exit when you press escape.
+#
+listShortcutsModeEnabled = False
+
 # The location of the user's preferences. By default this is ~/.orca.
 # It can be overridden by the Orca -d command line option.
 #
